@@ -14,7 +14,7 @@
 
 The test case encapsulates the request headers and body and expected
 response headers and body. When the test is run an HTTP request is
-made using urllib3. Assertions are made against the reponse.
+made using urllib3. Assertions are made against the response.
 """
 
 from collections import OrderedDict
@@ -23,14 +23,14 @@ import functools
 import os
 import re
 import sys
-from testtools import testcase
 import time
-from unittest import result
+from unittest import result as unitresult
 
 import six
 from six.moves import http_cookies
 from six.moves.urllib import parse as urlparse
 import testtools
+from testtools import testcase
 import wsgi_intercept
 
 from gabbi import __version__
@@ -69,6 +69,7 @@ BASE_TEST = {
     'xfail': False,
     'skip': '',
     'poll': {},
+    'use_prior_test': True,
 }
 
 
@@ -134,10 +135,11 @@ class HTTPTestCase(testtools.TestCase):
         if self.test_data['skip']:
             self.skipTest(self.test_data['skip'])
 
-        if self.prior and not self.prior.has_run:
+        if (self.prior and not self.prior.has_run and
+                self.test_data['use_prior_test']):
             # Use a different result so we don't count this test
             # in the results.
-            self.prior.run(result.TestResult())
+            self.prior.run(unitresult.TestResult())
         self._run_test()
 
     def get_content_handler(self, content_type):
@@ -147,15 +149,17 @@ class HTTPTestCase(testtools.TestCase):
                 return handler
         return None
 
-    def replace_template(self, message):
+    def replace_template(self, message, escape_regex=False):
         """Replace magic strings in message."""
         if isinstance(message, dict):
             for k in message:
-                message[k] = self.replace_template(message[k])
+                message[k] = self.replace_template(message[k],
+                                                   escape_regex=escape_regex)
             return message
 
         if isinstance(message, list):
-            return [self.replace_template(line) for line in message]
+            return [self.replace_template(line, escape_regex=escape_regex)
+                    for line in message]
 
         for replacer in REPLACERS:
             template = '$%s' % replacer
@@ -163,7 +167,8 @@ class HTTPTestCase(testtools.TestCase):
             try:
                 if template in message:
                     try:
-                        message = getattr(self, method)(message)
+                        replace = getattr(self, method)
+                        message = replace(message, escape_regex=escape_regex)
                     except (KeyError, AttributeError, ValueError) as exc:
                         raise AssertionError(
                             'unable to replace %s in %s, data unavailable: %s'
@@ -193,14 +198,28 @@ class HTTPTestCase(testtools.TestCase):
         value = value.encode('UTF-8')
         return value
 
-    def _environ_replace(self, message):
+    @staticmethod
+    def _regex_replacer(replacer, escape_regex):
+        """Wrap a replacer function to escape return values in a regex."""
+        if escape_regex:
+            @functools.wraps(replacer)
+            def replace(match):
+                return re.escape(replacer(match))
+
+            return replace
+        else:
+            return replacer
+
+    def _environ_replace(self, message, escape_regex=False):
         """Replace an indicator in a message with the environment value.
 
         If value can be a number, cast it as such. If value is a form of
         "null", "true", or "false" cast it to None, True, False.
         """
         value = re.sub(self._replacer_regex('ENVIRON'),
-                       self._environ_replacer, message)
+                       self._regex_replacer(self._environ_replacer,
+                                            escape_regex),
+                       message)
         try:
             if '.' in value:
                 value = float(value)
@@ -226,13 +245,15 @@ class HTTPTestCase(testtools.TestCase):
         environ_name = match.group('arg')
         return os.environ[environ_name]
 
-    def _cookie_replace(self, message):
+    def _cookie_replace(self, message, escape_regex=False):
         """Replace $COOKIE in a message.
 
         With cookie data from set-cookie in the prior request.
         """
         return re.sub(self._simple_replacer_regex('COOKIE'),
-                      self._cookie_replacer, message)
+                      self._regex_replacer(self._cookie_replacer,
+                                           escape_regex),
+                      message)
 
     def _cookie_replacer(self, match):
         """Replace a regex match with the cookie of a previous response."""
@@ -247,12 +268,14 @@ class HTTPTestCase(testtools.TestCase):
         cookie_string = cookies.output(attrs=[], header='', sep=',').strip()
         return cookie_string
 
-    def _headers_replace(self, message):
+    def _headers_replace(self, message, escape_regex=False):
         """Replace a header indicator in a message with that headers value from
         the prior request.
         """
         return re.sub(self._replacer_regex('HEADERS'),
-                      self._header_replacer, message)
+                      self._regex_replacer(self._header_replacer,
+                                           escape_regex),
+                      message)
 
     def _header_replacer(self, match):
         """Replace a regex match with the value of a prior header."""
@@ -264,20 +287,25 @@ class HTTPTestCase(testtools.TestCase):
             referred_case = self.prior
         return referred_case.response[header_key.lower()]
 
-    def _last_url_replace(self, message):
+    def _last_url_replace(self, message, escape_regex=False):
         """Replace $LAST_URL in a message.
 
         With the URL used in the prior request.
         """
-        return message.replace('$LAST_URL', self.prior.url)
+        last_url = self.prior.url
+        if escape_regex:
+            last_url = re.escape(last_url)
+        return message.replace('$LAST_URL', last_url)
 
-    def _url_replace(self, message):
+    def _url_replace(self, message, escape_regex=False):
         """Replace $URL in a message.
 
         With the URL used in a previous request.
         """
         return re.sub(self._simple_replacer_regex('URL'),
-                      self._url_replacer, message)
+                      self._regex_replacer(self._url_replacer,
+                                           escape_regex),
+                      message)
 
     def _url_replacer(self, match):
         """Replace a regex match with the value of a previous url."""
@@ -288,13 +316,15 @@ class HTTPTestCase(testtools.TestCase):
             referred_case = self.prior
         return referred_case.url
 
-    def _location_replace(self, message):
+    def _location_replace(self, message, escape_regex=False):
         """Replace $LOCATION in a message.
 
         With the location header from a previous request.
         """
         return re.sub(self._simple_replacer_regex('LOCATION'),
-                      self._location_replacer, message)
+                      self._regex_replacer(self._location_replacer,
+                                           escape_regex),
+                      message)
 
     def _location_replacer(self, match):
         """Replace a regex match with the value of a previous location."""
@@ -317,11 +347,13 @@ class HTTPTestCase(testtools.TestCase):
         with open(path, mode='rb') as data_file:
             return data_file.read()
 
-    def _netloc_replace(self, message):
+    def _netloc_replace(self, message, escape_regex=False):
         """Replace $NETLOC with the current host and port."""
         netloc = self.netloc
         if self.prefix:
             netloc = '%s%s' % (netloc, self.prefix)
+        if escape_regex:
+            netloc = re.escape(netloc)
         return message.replace('$NETLOC', netloc)
 
     def _parse_url(self, url):
@@ -371,7 +403,7 @@ class HTTPTestCase(testtools.TestCase):
         case = HTTPTestCase._history_regex
         return r"%s\$%s" % (case, key)
 
-    def _response_replace(self, message):
+    def _response_replace(self, message, escape_regex=False):
         """Replace a content path with the value from a previous response.
 
         If the match would replace the entire message, then don't cast it
@@ -381,7 +413,10 @@ class HTTPTestCase(testtools.TestCase):
         match = re.match('^%s$' % regex, message)
         if match:
             return self._response_replacer(match, preserve=True)
-        return re.sub(regex, self._response_replacer, message)
+        return re.sub(regex,
+                      self._regex_replacer(self._response_replacer,
+                                           escape_regex),
+                      message)
 
     def _response_replacer(self, match, preserve=False):
         """Replace a regex match with the value from a previous response."""
@@ -443,6 +478,21 @@ class HTTPTestCase(testtools.TestCase):
             self.response_data = None
         self.output = decoded_output
 
+    def _replace_headers_template(self, test_name, headers):
+        replaced_headers = {}
+
+        try:
+            for name in headers:
+                replaced_name = self.replace_template(name)
+                replaced_headers[replaced_name] = self.replace_template(
+                    headers[name]
+                )
+        except TypeError as exc:
+            raise exception.GabbiFormatError(
+                'malformed headers in test %s: %s' % (test_name, exc))
+
+        return replaced_headers
+
     def _run_test(self):
         """Make an HTTP request and compare the response with expectations."""
         test = self.test_data
@@ -452,14 +502,15 @@ class HTTPTestCase(testtools.TestCase):
         self.url = base_url
         full_url = self._parse_url(base_url)
 
+        # Replace variables in headers with variable values. This includes both
+        # in the header key and the header value.
+        test['request_headers'] = self._replace_headers_template(
+            test['name'], test['request_headers'])
+        test['response_headers'] = self._replace_headers_template(
+            test['name'], test['response_headers'])
+
         method = test['method'].upper()
         headers = test['request_headers']
-        for name in headers:
-            try:
-                headers[name] = self.replace_template(headers[name])
-            except TypeError as exc:
-                raise exception.GabbiFormatError(
-                    'malformed headers in test %s: %s' % (test['name'], exc))
 
         if test['data'] != '':
             body = self._test_data_to_string(
@@ -498,9 +549,10 @@ class HTTPTestCase(testtools.TestCase):
                               redirect=test['redirects'])
             self._assert_response()
 
-    def _scheme_replace(self, message):
+    def _scheme_replace(self, message, escape_regex=False):
         """Replace $SCHEME with the current protocol."""
-        return message.replace('$SCHEME', self.scheme)
+        scheme = re.escape(self.scheme) if escape_regex else self.scheme
+        return message.replace('$SCHEME', scheme)
 
     def _test_data_to_string(self, data, content_type):
         """Turn the request data into a string.
