@@ -19,6 +19,10 @@ import sys
 from unittest import case
 
 from gabbi import driver
+# TODO(cdent): test_pytest allows pytest to see the tests this module
+# produces. Without it, the generator will not run. It is a todo because
+# needing to do this is annoying and gross.
+from gabbi.driver import test_pytest  # noqa
 from gabbi import fixture
 
 
@@ -33,9 +37,19 @@ class LiveSkipFixture(fixture.GabbiFixture):
             raise case.SkipTest('live tests skipped')
 
 
+BUILD_TEST_ARGS = dict(
+    host='google.com',
+    fixture_module=sys.modules[__name__],
+    port=443
+)
+
+
 def load_tests(loader, tests, pattern):
     """Provide a TestSuite to the discovery process."""
     test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)
-    return driver.build_tests(test_dir, loader, host='google.com',
-                              fixture_module=sys.modules[__name__],
-                              port=443)
+    return driver.build_tests(test_dir, loader, **BUILD_TEST_ARGS)
+
+
+def pytest_generate_tests(metafunc):
+    test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)
+    driver.py_test_generator(test_dir, metafunc=metafunc, **BUILD_TEST_ARGS)
