@@ -12,8 +12,11 @@
 # under the License.
 """Utility methods shared by some tests."""
 
+import math
 import os
 
+import yaml
+
 
 def set_test_environ():
     """Set some environment variables used in tests."""
@@ -27,3 +30,13 @@ def set_test_environ():
     os.environ['FALSE'] = 'false'
     os.environ['STRING'] = 'val'
     os.environ['NULL'] = 'null'
+
+
+class NanChecker(yaml.YAMLObject):
+    yaml_tag = u'!NanChecker'
+
+    def __eq__(self, other):
+        try:
+            return math.isnan(other)
+        except TypeError:
+            return False
